using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Xml;

namespace XmlDemo
{
    public partial class frmCreateXml : Form
    {
        public List<Person> PeopleList = new List<Person>();
        XmlDocument doc = new XmlDocument();
        public frmCreateXml()
        {
            InitializeComponent();
            PeopleList.Add(new Person("Eric", 20, Gender.Male, "Hello", "USA"));
            PeopleList.Add(new Person("Kyle", 19, Gender.Male, "Goodbye", "USA"));
            PeopleList.Add(new Person("Daniel", 31, Gender.Male, "How are you?", "Great Britain"));
            PeopleList.Add(new Person("Heidi", 20, Gender.Female, "Hi", "USA"));
        }
        //Now we will create an Xml file using the list People, it is almost exactly the same as Writing to an Xml file
        public void CreateXml()
        {
            //create the declaration
            XmlDeclaration Declaration = doc.CreateXmlDeclaration("1.0", "utf-8", "");
            doc.AppendChild(Declaration);

            XmlNode People = doc.CreateNode(XmlNodeType.Element, "People", "");

            //Create a comment
            XmlComment Comment = doc.CreateComment("This is the new file!");
            //add it to the root
            People.AppendChild(Comment);

            //cycle through each person and create them
            PeopleList.ForEach(delegate(Person p)
            {
                XmlNode PersonNode = doc.CreateNode(XmlNodeType.Element, "Person", "");

                //The name of that person
                XmlNode nameNode = doc.CreateNode(XmlNodeType.Element, "Name", "");
                //set the name
                nameNode.InnerText = p.Name;
                //add it to the Person
                PersonNode.AppendChild(nameNode);

                //do the same for the rest of the nodes
                XmlNode ageNode = doc.CreateNode(XmlNodeType.Element, "Age", "");
                ageNode.InnerText = p.Age.ToString();
                PersonNode.AppendChild(ageNode);

                XmlNode countryNode = doc.CreateNode(XmlNodeType.Element, "Country", "");
                countryNode.InnerText = p.Country;
                PersonNode.AppendChild(countryNode);

                XmlNode genderNode = doc.CreateNode(XmlNodeType.Element, "Gender", "");
                genderNode.InnerText = p.Gender.ToString();
                PersonNode.AppendChild(genderNode);

                //create a new XmlAttribute for the message
                XmlAttribute messageAttribute = doc.CreateAttribute("message");
                messageAttribute.Value = p.Message;
                //add it to the PersonNodes' attributes
                PersonNode.Attributes.Append(messageAttribute);

                //add the person to the root node
                People.AppendChild(PersonNode);

            });
            //add the root node to the document
            doc.AppendChild(People);
            //save it
            doc.Save("NewXmlFile.xml");
        }
        public void ShowXml()
        {
            System.Diagnostics.ProcessStartInfo p = new System.Diagnostics.ProcessStartInfo("c:\\windows\\NOTEPAD.EXE", Application.StartupPath + "\\NewXmlFile.xml");
            System.Diagnostics.Process.Start(p);
        }

        public enum Gender
        {
            Male, Female, Undefined
        }
        public class Person
        {
            private string m_Name;
            public string Name
            {
                get { return m_Name; }
            }
            private int m_Age;
            public int Age
            {
                get { return m_Age; }
            }
            private Gender m_Gender;
            public Gender Gender
            {
                get { return m_Gender; }
            }
            private string m_Message;
            public string Message
            {
                get { return m_Message; }
            }
            private string m_Country;
            public string Country
            {
                get { return m_Country; }
            }
            public Person(string name, int age, Gender gender, string message, string country)
            {
                m_Name = name;
                m_Age = age;
                m_Gender = gender;
                m_Message = message;
                m_Country = country;
            }
        }

        private void frmCreateXml_FormClosing(object sender, FormClosingEventArgs e)
        {
            e.Cancel = true;
            this.Hide();
        }

        private void btnCreate_Click(object sender, EventArgs e)
        {
            label1.Text = "Created Xml...";
            CreateXml();
            label1.Text = "Opening Xml...";
            ShowXml();
            label1.Text = "Opened...";
        }
    }
}