using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Xml;

namespace XmlDemo
{
    public partial class frmReadXml : Form
    {
        public XmlDocument doc = new XmlDocument();
        public List<Person> People = new List<Person>();
        public frmReadXml()
        {
            InitializeComponent();
        }

        private void frmDeserialize_FormClosing(object sender, FormClosingEventArgs e)
        {
            e.Cancel = true;
            this.Hide();
        }
        private void PopulatePeople()
        {
            //since we know the layout of the Xml we can find the nodes using their XPath
            //this is the root node
            XmlNode RootNode = doc.SelectSingleNode("People");//When retrieve nodes or attributes, the names are case sensitive so "People" is not the same as "people"
            //since we know what is in this Xml file, we can grab a NodeList directly without using the RootNode
            XmlNodeList PersonNodeList = doc.SelectNodes("People/Person");//this will put every Person node into a list

            /**This list is similar to an array and you can access one of the XmlNodes directly like so
             * 
             * PersonNodeList.Item(1)
             * 
             */
            
            //or can iterated through like so
            foreach (XmlNode PersonNode in PersonNodeList)//this will cycle through every Person node
            {
                //now that we have access to a node, lets pull its information, create a Person class and add it to the list
                string name, message, country;
                int age;
                Gender gender;

                /*note that if the attribute does not exist, it will throw an null reference exception
                    so you need to use ty{} catch{} blocks or check if PersonNode.Attributes["name"] is null
                 */
                if (PersonNode.Attributes["message"] != null)
                {
                    //You can retrieve the attributes of a node by either the name of the attribute
                    message = PersonNode.Attributes["message"].Value;
                    //by index
                    //PersonNode.Attributes[0].Value
                }
                else
                {
                    message = String.Empty;
                }

                //for retieving nodes inside of this one (which is "People/Person") we use this
                XmlNode NameNode = PersonNode.SelectSingleNode("Name");//this is ultimately "People/Person/Name" 
                //Like attributes, we must check if NameNode exists, if not it will throw a null reference exception, we will use a try{} catch{} here
                //a node doesnt exist if the tags arent there
                try
                {
                    name = NameNode.InnerText;//InnerText is the text between two tags
                }
                catch
                {
                    MessageBox.Show("This person has no name tag!!!");
                    name = String.Empty;
                }

                //Although creating a new XmlNode makes code a bit easier to read and understand, it seems like a waste of time
                //we could also do this
                try
                {
                    country = PersonNode.SelectSingleNode("Country").InnerText;//this is the same as creating a new XmlNode, but much quicker
                }
                catch
                {
                    MessageBox.Show("This person has no country tag!!!");
                    country = String.Empty;
                }

                //Since everything retrieved is a string, you must parse them to the correct data type

                if (PersonNode.SelectSingleNode("Age") != null)
                {
                    if (!int.TryParse(PersonNode.SelectSingleNode("Age").InnerText, out age))//use TryParse
                    {
                        //if it wasnt an integer
                        MessageBox.Show("This person did not have a integer for an age!!");
                        age = 0;
                    }
                }
                else
                {
                    MessageBox.Show("This person did not have an age tag!!!");
                    age = 0;
                }
                //you could use this same method for bool, etc as well
                //for enums, you can use a switch statement
                if (PersonNode.SelectSingleNode("Gender") != null)
                {
                    switch (PersonNode.SelectSingleNode("Gender").InnerText)
                    {
                        case "Male":
                            gender = Gender.Male;
                            break;
                        case "Female":
                            gender = Gender.Female;
                            break;
                        default:
                            MessageBox.Show("This person does not have a correct gender!!");
                            gender = Gender.Undefined;
                            break;
                    }
                }
                else
                {
                    MessageBox.Show("This person did not have a gender tag!!!");
                    gender = Gender.Undefined;
                }

                //create the new Person
                Person p = new Person(name, age, gender, message, country);
                //add them to People
                People.Add(p);
            }
        }
        private void GenerateText()
        {
            People.ForEach(delegate(Person p)
            {
                textBox.Text += String.Format("Name: {0}\r\nAge: {1}\r\nCountry: {2}\r\nGender: {3}\r\nMessage: {4}\r\n----------\r\n", p.Name, p.Age, p.Country, p.Gender, p.Message);
            });
            textBox.Select(0, 0);
        }
        private void frmDeserialize_VisibleChanged(object sender, EventArgs e)
        {
            if (frmMain.XmlFile != String.Empty)
            {
                //load the xml file into the XmlDocument
                doc.Load(frmMain.XmlFile);
                PopulatePeople();//Create each Person and add them to People
                GenerateText();//Write out each Person's information
            }
        }

        private void frmReadXml_Load(object sender, EventArgs e)
        {

        }
    }
    public enum Gender
    {
        Male, Female, Undefined
    }
    public class Person
    {
        private string m_Name;
        public string Name
        {
            get { return m_Name; }
        }
        private int m_Age;
        public int Age
        {
            get { return m_Age; }
        }
        private Gender m_Gender;
        public Gender Gender
        {
            get { return m_Gender; }
        }
        private string m_Message;
        public string Message
        {
            get { return m_Message; }
        }
        private string m_Country;
        public string Country
        {
            get { return m_Country; }
        }
        public Person(string name, int age, Gender gender, string message, string country)
        {
            m_Name = name;
            m_Age = age;
            m_Gender = gender;
            m_Message = message;
            m_Country = country;
        }
    }
}