using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Xml;

namespace XmlDemo
{
    public partial class frmWriteXml : Form
    {
        public XmlDocument doc = new XmlDocument();
        public frmWriteXml()
        {
            InitializeComponent();
        }
        //Saving changes to current nodes is extremely easily
        private void SaveNode()
        {
            //load the node from the document based on the combobox selection
            XmlNode PersonNode = doc.SelectNodes("People/Person").Item(comboBox.SelectedIndex);
            //go through each of the child nodes and change their text
            PersonNode.SelectSingleNode("Name").InnerText = txtName.Text;
            PersonNode.SelectSingleNode("Age").InnerText = txtAge.Text;
            PersonNode.SelectSingleNode("Country").InnerText = txtCountry.Text;
            PersonNode.SelectSingleNode("Gender").InnerText = cmboGender.SelectedItem.ToString();
            //change the attribute value to the text
            PersonNode.Attributes["message"].Value = txtMessage.Text;

            //save the file
            doc.Save(frmMain.XmlFile);

            //thats all there is to save changes to a previously created node

            //repopulate the combobox
            PopulateComboBox();
        }
        //Creating a new XmlNode is pretty easy as well (although a bit tedious)
        private void CreateNew()
        {
            //Create an XmlNode for each node, you use the XmlDocument to create each new one

            //The new Person that we are adding
            XmlNode PersonNode = doc.CreateNode(XmlNodeType.Element, "Person", "");

            //The name of that person
            XmlNode nameNode = doc.CreateNode(XmlNodeType.Element, "Name", "");
            //set the name
            nameNode.InnerText = txtName.Text;
            //add it to the Person
            PersonNode.AppendChild(nameNode);

            //do the same for the rest of the nodes
            XmlNode ageNode = doc.CreateNode(XmlNodeType.Element, "Age", "");
            ageNode.InnerText = txtAge.Text;
            PersonNode.AppendChild(ageNode);

            XmlNode countryNode = doc.CreateNode(XmlNodeType.Element, "Country", "");
            countryNode.InnerText = txtCountry.Text;
            PersonNode.AppendChild(countryNode);

            XmlNode genderNode = doc.CreateNode(XmlNodeType.Element, "Gender", "");
            genderNode.InnerText = cmboGender.SelectedItem.ToString();
            PersonNode.AppendChild(genderNode);

            //create a new XmlAttribute for the message
            XmlAttribute messageAttribute = doc.CreateAttribute("message");
            messageAttribute.Value = txtMessage.Text;
            //add it to the PersonNodes' attributes
            PersonNode.Attributes.Append(messageAttribute);

            //Add this new person into document
            doc.DocumentElement.AppendChild(PersonNode);
            //save it
            doc.Save(frmMain.XmlFile);

            //repopulate the combobox
            PopulateComboBox();
        }

        private void btnCreate_Click(object sender, EventArgs e)
        {
            CreateNew();
        }

        private void frmWriteXml_VisibleChanged(object sender, EventArgs e)
        {
            if (frmMain.XmlFile != String.Empty)
            {
                //load the xml file into the XmlDocument
                doc.Load(frmMain.XmlFile);
                PopulateComboBox();
            }
        }
        private void PopulateComboBox()
        {
            comboBox.Items.Clear();
            foreach (XmlNode PersonNode in doc.SelectNodes("People/Person"))//this will select all the Person nodes
            {
                comboBox.Items.Add(PersonNode.SelectSingleNode("Name").InnerText);
            }
        }

        private void comboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            XmlNode PersonNode = doc.SelectNodes("People/Person").Item(comboBox.SelectedIndex);
            Person p = CreatePerson(PersonNode);
            if (p != null)
            {
                txtName.Text = p.Name;
                txtAge.Text = p.Age.ToString();
                txtCountry.Text = p.Country;
                txtMessage.Text = p.Message;
                cmboGender.SelectedItem = p.Gender.ToString();
            }
        }

        //this is just a copy of the stuff from ReadXml, so look for comments there
        private Person CreatePerson(XmlNode PersonNode)
        {
            string name, message, country;
            int age;
            Gender gender;
            if (PersonNode == null)
                return null;

            if (PersonNode.Attributes["message"] != null)
            {
                message = PersonNode.Attributes["message"].Value;
            }
            else
            {
                message = String.Empty;
            }

            XmlNode NameNode = PersonNode.SelectSingleNode("Name");
            try
            {
                name = NameNode.InnerText;
            }
            catch
            {
                MessageBox.Show("This person has no name tag!!!");
                name = String.Empty;
            }

            try
            {
                country = PersonNode.SelectSingleNode("Country").InnerText;
            }
            catch
            {
                MessageBox.Show("This person has no country tag!!!");
                country = String.Empty;
            }


            if (PersonNode.SelectSingleNode("Age") != null)
            {
                if (!int.TryParse(PersonNode.SelectSingleNode("Age").InnerText, out age))
                {
                    MessageBox.Show("This person did not have a integer for an age!!");
                    age = 0;
                }
            }
            else
            {
                MessageBox.Show("This person did not have an age tag!!!");
                age = 0;
            }

            if (PersonNode.SelectSingleNode("Gender") != null)
            {
                switch (PersonNode.SelectSingleNode("Gender").InnerText)
                {
                    case "Male":
                        gender = Gender.Male;
                        break;
                    case "Female":
                        gender = Gender.Female;
                        break;
                    default:
                        MessageBox.Show("This person does not have a correct gender!!");
                        gender = Gender.Undefined;
                        break;
                }
            }
            else
            {
                MessageBox.Show("This person did not have a gender tag!!!");
                gender = Gender.Undefined;
            }

            Person p = new Person(name, age, gender, message, country);
            return p;
        }

        private void btnSave_Click(object sender, EventArgs e)
        {
            SaveNode();
        }

        private void frmWriteXml_FormClosing(object sender, FormClosingEventArgs e)
        {
            e.Cancel = true;
            this.Hide();
        }

        private void frmWriteXml_Load(object sender, EventArgs e)
        {

        }
    }
}