using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Xml;

namespace XmlDemo
{
    /// <summary>
    /// This will demostrate how to iterate through every thing in the xmlfile without knowing its contents
    /// </summary>
    public partial class frmXmlTree : Form
    {
        public XmlDocument doc = new XmlDocument();
        public frmXmlTree()
        {
            InitializeComponent();
        }
        /// <summary>
        /// Load the Xml document into the XmlTree
        /// </summary>
        private void PopulateXmlTree()
        {
            XmlTree.Nodes.Add(PopulateNode(doc.DocumentElement));
        }
        private TreeNode PopulateNode(XmlNode XmlNode)
        {
            List<TreeNode> ChildTreeNodes = new List<TreeNode>();
            //cycle through an attributes
            foreach (XmlAttribute attribute in XmlNode.Attributes)
            {
                TreeNode treeNode = new TreeNode("Attribute: " + attribute.Name + "=\"" + attribute.Value + "\"");
                ChildTreeNodes.Add(treeNode);
            }
            //cycle through each child node
            foreach (XmlNode ChildXmlNode in XmlNode.ChildNodes)
            {
                //if there is no more child nodes, then find the value
                if (ChildXmlNode.ChildNodes.Count == 0)
                {
                    TreeNode ChildNode = new TreeNode();
                    if (ChildXmlNode is XmlComment)//if it is a comment, lets make sure to specify
                    {
                        ChildNode.Text = "Comment: " + ChildXmlNode.Value;
                        //go through any attributes
                        if (ChildXmlNode.Attributes != null)
                        {
                            foreach (XmlAttribute attribute in ChildXmlNode.Attributes)
                            {
                                TreeNode treeNode = new TreeNode("Attribute: " + attribute.Name + "=\"" + attribute.Value + "\"");
                                ChildNode.Nodes.Add(treeNode);
                            }
                        }
                        ChildTreeNodes.Add(ChildNode);
                    }
                    else if (ChildXmlNode.InnerText != String.Empty)//check if if has InnerText
                    {
                        ChildNode.Text = ChildXmlNode.Value;
                        //go through any attributes
                        if (ChildXmlNode.Attributes != null)
                        {
                            foreach (XmlAttribute attribute in ChildXmlNode.Attributes)
                            {
                                TreeNode treeNode = new TreeNode("Attribute: " + attribute.Name + "=\"" + attribute.Value + "\"");
                                ChildNode.Nodes.Add(treeNode);
                            }
                        }
                        ChildTreeNodes.Add(ChildNode);
                    }
                    else//just add the name if it is empty tags
                    {
                        ChildNode.Text = ChildXmlNode.Name;
                        //go through any attributes
                        if (ChildXmlNode.Attributes != null)
                        {
                            foreach (XmlAttribute attribute in ChildXmlNode.Attributes)
                            {
                                TreeNode treeNode = new TreeNode("Attribute: " + attribute.Name + "=\"" + attribute.Value + "\"");
                                ChildNode.Nodes.Add(treeNode);
                            }
                        }
                        ChildTreeNodes.Add(ChildNode);
                    }
                }
                //if there are some more children nodes, lets add the node and cycle through it 
                else
                {
                    ChildTreeNodes.Add(PopulateNode(ChildXmlNode));
                }
            }
            TreeNode TreeNode = new TreeNode(XmlNode.Name, ChildTreeNodes.ToArray());
            return TreeNode;
        }

        private void frmXmlTree_FormClosing(object sender, FormClosingEventArgs e)
        {
            e.Cancel = true;
            this.Hide();
        }

        private void frmXmlTree_VisibleChanged(object sender, EventArgs e)
        {
            if (frmMain.XmlFile != String.Empty)
            {
                //load the xml file into the XmlDocument
                doc.Load(frmMain.XmlFile);
                PopulateXmlTree();
            }
        }
    }
}